//+------------------------------------------------------------------+
//|                                                  GomezEMP_TF.mq4 |
//|                                 Copyright  2011, Andrew Dossett |
//|                     fxprogrammer@rambler.ru, http://fxcoding.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Andrew Dossett"
#property link      "http://fxcoding.com, fxprogrammer@rambler.ru"

extern bool   MoneyManagement=true;
extern double Risk=2.5;
int Slippage=5;            // pips slippage allowed
extern int    breakeven = 0;
extern double trail_start = 20; 
extern double trail_amount = 20;
extern bool   UseManualSL=false;
extern int manual_stoploss = 0;
extern bool use_both_trades = true;
extern int TakeProfit1=20;         // pips take profit order #1
extern int TakeProfit2=30;         // pips take profit order #2
extern int TakeProfit3=40;         // pips take profit order #2
extern double Lot1=0.01;          // lots to trade (fractional values ok)
extern double Lot2=0.01;          // lots to trade (fractional values ok)
extern double Lot3=0.01;          // lots to trade (fractional values ok)

extern string Time_Settings = "===== Time Settings =====";
extern bool use_time_filter = false;
extern int  startHour = 7;          // Start of trading session
extern int  endHour = 17;          // end of trading session

extern string BBAnds="BBands parameters";
extern int    Length=20;      // Bollinger Bands Period
extern double Deviation=2;    // Deviation
extern double MoneyRisk=1.00; // Offset Factor
extern int    Signal=1;       // Display signals mode: 1-Signals & Stops; 0-only Stops; 2-only Signals;
extern int    Line=1;         // Display line mode: 0-no,1-yes  
extern int    Nbars=1000;

int magic1=1234;                  // order #1's magic number
int magic2=4321;                  // order #2's magic number
int magic3=4675;                  // order #3's magic number

double long_signal;
double short_signal;
datetime last_bar = 0;

//===========================================================================================
//===========================================================================================

int init()
{
   if (Point == 0.001 || Point == 0.00001) {
     manual_stoploss = manual_stoploss*10;
     TakeProfit1 = TakeProfit1*10;
     TakeProfit2 = TakeProfit2*10;
     TakeProfit3 = TakeProfit3*10;
     Slippage = Slippage*10;
   }
 return(0);
}

int deinit()
{
 return(0);
}

//===========================================================================================
//===========================================================================================

int start()     
{
   if(OrdersTotal() != 0) trailing(); // trailing function
   if(OrdersTotal() != 0) BE(); // breakeven

   if (!time_filter()) return;   
   if (last_bar == iTime(NULL, 0, 0)) return;

   //money management measuring
   if (MoneyManagement)
   {
      int Leverage=AccountLeverage();
      double Lots=MathFloor((AccountBalance()*Risk*(Leverage/100))/(MarketInfo(Symbol(),MODE_LOTSIZE)*MarketInfo(Symbol(),MODE_MINLOT)))*MarketInfo(Symbol(),MODE_MINLOT);
      Lot1 = Lots;
      Lot2 = Lots;
      Lot3 = Lots;
   }


   if (!use_both_trades) {
      if(OrdersTotal() == 0) {
         long_signal = iCustom(NULL, 0, "BBands_Stop_v1", Length, Deviation, MoneyRisk, Signal, Line, Nbars, 2, 1);
         //Print ("Long = ", long_signal, ", Short = ", short_signal);
         if (long_signal != -1) {
             OpenLong(long_signal);
             last_bar = iTime(NULL, 0, 0);
             return;
         }
         short_signal = iCustom(NULL, 0, "BBands_Stop_v1", Length, Deviation, MoneyRisk, Signal, Line, Nbars, 3, 1);      
         if (short_signal != -1) {
             OpenShort(short_signal);
             last_bar = iTime(NULL, 0, 0);
             return;
         }      
      }
   } else {
         long_signal = iCustom(NULL, 0, "BBands_Stop_v1", Length, Deviation, MoneyRisk, Signal, Line, Nbars, 2, 1);
         //Print ("Long = ", long_signal, ", Short = ", short_signal);
         if (long_signal != -1) {
             OpenLong(long_signal);
             last_bar = iTime(NULL, 0, 0);
             return;
         }
         short_signal = iCustom(NULL, 0, "BBands_Stop_v1", Length, Deviation, MoneyRisk, Signal, Line, Nbars, 3, 1);      
         if (short_signal != -1) {
             OpenShort(short_signal);
             last_bar = iTime(NULL, 0, 0);
             return;
         }    
   }

   last_bar = iTime(NULL, 0, 0);
}

//===========================================================================================
//===========================================================================================

//===========================================================================================
//===========================================================================================

bool time_filter() {
   if (!use_time_filter) return (true);
   if (Hour() >= startHour && Hour() < endHour) return (true);
   return(false);
}
 
//===========================================================================================
//===========================================================================================

void OpenLong(double sl) {    
   if (UseManualSL) {
     if (manual_stoploss > 0) sl = NormalizeDouble(Ask - manual_stoploss*Point,Digits);
     if (manual_stoploss == 0) sl = 0;
   }
   if (Lot1 != 0) OrderSend(Symbol(),OP_BUY,Lot1,NormalizeDouble(Ask,Digits),Slippage,NormalizeDouble(sl,Digits),NormalizeDouble(Ask + TakeProfit1*Point,Digits),NULL,magic1,0,Blue);
   if (Lot2 != 0) OrderSend(Symbol(),OP_BUY,Lot2,NormalizeDouble(Ask,Digits),Slippage,NormalizeDouble(sl,Digits),NormalizeDouble(Ask + TakeProfit2*Point,Digits),NULL,magic2,0,Blue);
   if (Lot3 != 0) OrderSend(Symbol(),OP_BUY,Lot3,NormalizeDouble(Ask,Digits),Slippage,NormalizeDouble(sl,Digits),NormalizeDouble(Ask + TakeProfit3*Point,Digits),NULL,magic3,0,Blue);
   return;
}

//===========================================================================================
//===========================================================================================

void OpenShort(double sl) {    
   if (UseManualSL) {
     if (manual_stoploss > 0) sl = NormalizeDouble(Bid + manual_stoploss*Point,Digits);
     if (manual_stoploss == 0) sl = 0;
   }
   if (Lot1 != 0) OrderSend(Symbol(),OP_SELL,Lot1,NormalizeDouble(Bid,Digits),Slippage,NormalizeDouble(sl,Digits),NormalizeDouble(Bid - TakeProfit1*Point,Digits),NULL,magic1,0,Red);
   if (Lot2 != 0) OrderSend(Symbol(),OP_SELL,Lot2,NormalizeDouble(Bid,Digits),Slippage,NormalizeDouble(sl,Digits),NormalizeDouble(Bid - TakeProfit2*Point,Digits),NULL,magic2,0,Red);
   if (Lot3 != 0) OrderSend(Symbol(),OP_SELL,Lot3,NormalizeDouble(Bid,Digits),Slippage,NormalizeDouble(sl,Digits),NormalizeDouble(Bid - TakeProfit3*Point,Digits),NULL,magic3,0,Red);
   return;
}

//===========================================================================================
//===========================================================================================

void trailing() {
   if (trail_start == 0) return;
   double pip = Point;
   if (pip == 0.001 || pip == 0.00001) pip = pip*10;   
   for(int cnt = 0; cnt < OrdersTotal(); cnt++) {
     OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
        if(OrderSymbol() == Symbol() && (OrderMagicNumber() == magic1 || OrderMagicNumber() == magic2 || OrderMagicNumber() == magic3)) {
           if (OrderType()==OP_BUY && OrderStopLoss() >= OrderOpenPrice()) { 
              if (Bid > OrderStopLoss() + pip*trail_start + pip*trail_amount)
                   OrderModify(OrderTicket(),OrderOpenPrice(),Bid - pip*trail_start,OrderTakeProfit(),0,Green);
           }
           if (OrderType()==OP_SELL && OrderStopLoss() >= OrderOpenPrice()) {
              if (Ask < OrderStopLoss() - pip*trail_start - pip*trail_amount)
                  OrderModify(OrderTicket(),OrderOpenPrice(),Ask + pip*trail_start,OrderTakeProfit(),0,Red);
           }
       }
   }    
   return;
}

//===========================================================================================
//===========================================================================================

void BE() { // break even
  if (breakeven == 0) return;
  double pip = Point;
  if (pip == 0.001 || pip == 0.00001) pip = pip*10; 
  for(int i=0;i<OrdersTotal();i++) { 
    OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
    if (OrderSymbol() == Symbol() && (OrderMagicNumber() == magic1 || OrderMagicNumber() == magic2 || OrderMagicNumber() == magic3)) {
      if (OrderType() == OP_BUY) {
         if (NormalizeDouble(OrderStopLoss(),Digits) != NormalizeDouble(OrderOpenPrice(),Digits))
             if (Bid > OrderOpenPrice() + breakeven*pip) 
                 OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0,Green);
      }     
      if (OrderType() == OP_SELL) {
         if (NormalizeDouble(OrderStopLoss(),Digits) != NormalizeDouble(OrderOpenPrice(),Digits))
           if (Ask < OrderOpenPrice() - breakeven*pip) 
              OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0,Red);
      }
    }
  }          
  return;
} 